@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 
-# $Id: ClamdOmitScan.pl,v 1.15 2007/09/23 03:05:10 okamura Exp $
+# $Id: ClamdOmitScan.pl,v 1.28 2007/12/11 02:46:41 okamura Exp $
 
 require 5.8.0;
 use strict;
@@ -23,6 +23,7 @@ use Time::Local;
 eval {
 	require Sys::Filesystem;
 };
+use File::Temp qw(tempdir);
 
 # プロジェクト
 
@@ -49,7 +50,7 @@ use vars qw(
 $ProgramName = 'ClamdOmitScan';
 
 # バージョン
-$Version = 'ClamdOmitScan.pl v1.1.1';
+$Version = 'ClamdOmitScan.pl v1.2';
 
 # プレフィクス
 $Prefix = '/usr/local';
@@ -242,17 +243,46 @@ sub ReportResult {
 	}
 }
 
+# 出力ファイル名の変換
+# @param $path	実際にスキャンしたファイル名
+# @return 置き換え後のファイル名
+sub ReplaceFName {
+	my	($path) = @_;
+
+	if (
+		defined($ActionSwitch{targetPrefix})
+			and
+		defined($ActionSwitch{targetPrefix}->{prefix})
+			and
+		defined($ActionSwitch{targetPrefix}->{prefixLen})
+			and
+		defined($ActionSwitch{targetPrefix}->{replace})
+			and
+		substr($path, 0, $ActionSwitch{targetPrefix}->{prefixLen})
+			eq
+		$ActionSwitch{targetPrefix}->{prefix}
+	) {
+		$path = $ActionSwitch{targetPrefix}->{replace}
+				.
+			substr($path, $ActionSwitch{targetPrefix}->{prefixLen})
+		;
+	}
+
+	return $path;
+}
+
 # クリーン応答レポート
 # @param $path パス
 sub ReportOkResult {
 	my	($path) = @_;
+	my	$reportPath = ReplaceFName($path);
 
-	Output "%s: OK\n", EscapedPath($path)	unless ($Conf{infected});
+	Output "%s: OK\n", EscapedPath($reportPath)	unless ($Conf{infected});
 	$Summary{count}->{files}->{scaned}++;
 	$Summary{count}->{data}->{scaned}
 		+= $ActionSwitch{statCache}->[7]
 	;
-	UpdateScanData($ActionSwitch{ScanDataHandle}, $path);
+	UpdateScanData($ActionSwitch{ScanDataHandle}, $reportPath);
 }
 
 # 感染応答レポート
@@ -260,8 +290,9 @@ sub ReportOkResult {
 # @param $virus ウィルス名
 sub ReportFoundResult {
 	my	($path, $virus) = @_;
+	my	$reportPath = ReplaceFName($path);
 
-	Output "%s: %s FOUND\n", EscapedPath($path), $virus;
+	Output "%s: %s FOUND\n", EscapedPath($reportPath), $virus;
 	$Summary{count}->{files}->{scaned}++;
 	$Summary{count}->{data}->{scaned}
 		+= $ActionSwitch{statCache}->[7]
@@ -276,8 +307,9 @@ sub ReportFoundResult {
 # @param $message エラーメッセージ
 sub ReportErrorResult {
 	my	($path, $message) = @_;
+	my	$reportPath = ReplaceFName($path);
 
-	Output "%s: %s ERROR\n", EscapedPath($path), $message;
+	Output "%s: %s ERROR\n", EscapedPath($reportPath), $message;
 	$Summary{count}->{error}++;
 }
 
@@ -286,8 +318,9 @@ sub ReportErrorResult {
 # @param $message メッセージ
 sub ReportUnknownResult {
 	my	($path, $message) = @_;
+	my	$reportPath = ReplaceFName($path);
 
-	Output "%s: %s\n", EscapedPath($path), $message;
+	Output "%s: %s\n", EscapedPath($reportPath), $message;
 	$Summary{count}->{error}++;
 }
 
@@ -438,9 +471,56 @@ sub _SlimScanData {
 	my	$dataPath = $File::Find::name;
 	my	$realPath = substr($dataPath, $ActionSwitch{ScanDataBasePathLen});
 
+	if (defined $ActionSwitch{targetPrefix}) {
+		$realPath = ReplaceFName($realPath);
+	}
+
 	unless ($realPath eq '') {
 		if (-d $dataPath) {
-			unless (-d $realPath) {
+			if (-d $realPath) {
+				1;
+			}
+			elsif (-f $realPath and $Conf{dmg} && IsDiskImageFile($realPath)) {
+				my	$mountPoint;
+				my	@umountArgs;
+				my	$check;
+
+				$check = defined(
+					$mountPoint = tempdir('ClamdOmitScan-XXXXXXXXXX',
+						CLEANUP => 1, DIR => File::Spec->tmpdir
+					)
+				);
+				if ($check) {
+					$mountPoint = realpath($mountPoint);
+					$check =
+						0 < scalar(
+							@umountArgs = MountDiskImage(
+								$realPath, $mountPoint
+							)
+						)
+							and
+						$umountArgs[0] ? 1 : 0
+					;
+					if ($check) {
+						my	$nextRealPath = substr($dataPath, $ActionSwitch{ScanDataBasePathLen});
+
+						local	$ActionSwitch{targetPrefix} = {
+							prefix		=> $nextRealPath,
+							prefixLen	=> length($nextRealPath),
+							replace		=> $mountPoint,
+						};
+
+						find(\&_SlimScanData, $dataPath);
+						$check = UnmountDiskImage(@umountArgs);
+					}
+					rmdir $mountPoint;
+				}
+				unless ($check) {
+					Error "$!";
+				}
+				$File::Find::prune = 1;
+			}
+			else {
 				unless (rmtree($dataPath, 0, 0)) {
 					Warn '%s: %s', "$!", EscapedPath($dataPath);
 				}
@@ -448,7 +528,9 @@ sub _SlimScanData {
 			}
 		}
 		elsif (-f $dataPath) {
-			unless (-f $realPath) {
+			unless (
+				-f $realPath and !$Conf{dmg} || !IsDiskImageFile($realPath)
+			) {
 				unless (unlink($dataPath)) {
 					Warn '%s: %s', "$!", EscapedPath($dataPath);
 				}
@@ -468,11 +550,58 @@ sub _SlimScanData4Maintain {
 	my	$dataPath = $File::Find::name;
 	my	$realPath = substr($dataPath, $ActionSwitch{ScanDataBasePathLen});
 
+	if (defined $ActionSwitch{targetPrefix}) {
+		$realPath = ReplaceFName($realPath);
+	}
+
 	unless ($realPath eq '') {
 		if (-d $dataPath) {
 			if (-d $realPath) {
 				chmod $Conf{'perm-dir'}, $dataPath;
 			}
+			elsif (-f $realPath and $Conf{dmg} && IsDiskImageFile($realPath)) {
+				my	$mountPoint;
+				my	@umountArgs;
+				my	$check;
+
+				$check = defined(
+					$mountPoint = tempdir('ClamdOmitScan-XXXXXXXXXX',
+						CLEANUP => 1, DIR => File::Spec->tmpdir
+					)
+				);
+				if ($check) {
+					$mountPoint = realpath($mountPoint);
+					$check =
+						0 < scalar(
+							@umountArgs = MountDiskImage(
+								$realPath, $mountPoint
+							)
+						)
+							and
+						$umountArgs[0] ? 1 : 0
+					;
+					if ($check) {
+						local	$ActionSwitch{targetPrefix} = {
+							prefix		=> $realPath,
+							prefixLen	=> length($realPath),
+							replace		=> $mountPoint,
+						};
+
+						find(\&_SlimScanData4Maintain, $dataPath);
+						$check = UnmountDiskImage(@umountArgs);
+					}
+					else {
+						Error "Failed to mount a disk image file: %s",
+							$realPath
+						;
+					}
+					rmdir $mountPoint;
+				}
+				unless ($check) {
+					Error "$!";
+				}
+				$File::Find::prune = 1;
+			}
 			else {
 				unless (rmtree($dataPath, 0, 0)) {
 					Warn '%s: %s', "$!", EscapedPath($dataPath);
@@ -481,7 +610,7 @@ sub _SlimScanData4Maintain {
 			}
 		}
 		elsif (-f $dataPath) {
-			if (-f $realPath) {
+			if (-f $realPath and !$Conf{dmg} || !IsDiskImageFile($realPath)) {
 				my	%version = _LastScanClamavVersion($dataPath);
 
 				if ($version{ClamAV}) {
@@ -529,7 +658,9 @@ sub SlimScanData {
 			$scanDataDir = $handle->{basePath};
 		}
 		else {
-			$scanDataDir = File::Spec->catfile($handle->{basePath}, $dir);
+			$scanDataDir = File::Spec->catfile(
+				$handle->{basePath}, substr($dir, 1)
+			);
 		}
 	}
 	else {
@@ -553,6 +684,7 @@ sub LastScanedTime {
 
 	return undef	unless ($handle->{fileHandle});
 
+	$path = ReplaceFName($path);
 	$path = File::Spec->catfile($handle->{basePath}, $path);
 	return undef	unless (-e $path);
 
@@ -664,7 +796,7 @@ sub AddScanData {
 			chmod $Conf{'perm-dir'}, $_;
 		}
 	}
-	if ($ret and -f $path) {
+	if ($ret) {
 		# ファイル $scanData を作成する
 		my	$fh;
 
@@ -678,10 +810,6 @@ sub AddScanData {
 			$ret = 0;
 		}
 	}
-	else {
-		Warn 'Is not a file: %s', EscapedPath($path);
-		$ret = 0;
-	}
 
 	return $ret;
 }
@@ -731,6 +859,63 @@ sub UpdateScanData {
 
 
 #--- 初期化 ----
+# Darwin のファイルシステムの情報を更新する
+sub _UpdateFileSsystemInfo4darwin {
+	my	$fh;
+
+	Debug 'Get filesystem informations...';
+	if (open $fh, '-|', qw(/usr/sbin/disktool -l)) {
+		$ActionSwitch{filesystem} = [];
+		while (my $line = <$fh>) {
+			chomp $line;
+			unless (
+				$line =~ m/^\*\*\*Disk Appeared \((?#
+					)'.*',(?#
+					)Mountpoint = '(.*)', (?#
+					)fsType = '(.*)', (?#
+					)volName = '.*'\)$/
+			) {
+				Warn 'Unknown disktool output format: %s', $line;
+				next;
+			}
+			my	($mp, $ft) = ($1, $2);
+
+			next	if ($mp eq '');
+			Debug "\t%s: %s", $ft, $mp;
+			push @{$ActionSwitch{filesystem}}, {
+				mountPoint	=> $mp,
+				type		=> $ft
+			};
+		}
+		close $fh;
+		Debug 'done.';
+	}
+	else {
+		Error '/usr/sbin/disktool -l: %s', "$!"; 
+	}
+}
+
+# ファイルシステムの情報を更新する
+sub UpdateFileSystemInfo {
+	my	$fs = new Sys::Filesystem;
+
+	Debug 'Get filesystem informations...';
+	$ActionSwitch{filesystem} = [
+		map {
+			Debug "\t%s: %s", $fs->type($_), $fs->mount_point($_);
+			{
+				mountPoint	=> $fs->mount_point($_),
+				type		=> $fs->type($_),
+			};
+		} $fs->mounted_filesystems()
+	];
+	Debug 'done.';
+}
+
+if ($^O eq 'darwin') {
+	*UpdateFileSystemInfo = \*_UpdateFileSsystemInfo4darwin;
+}
+
 # 初期化
 # @return 成功/失敗
 sub Initialize {
@@ -769,23 +954,7 @@ sub Initialize {
 	}
 
 	if ($Conf{rsrc}) {
-		my	$fs = Sys::Filesystem->new();
-
-		$ActionSwitch{filesystem} = [
-			map {
-				{
-					mountPoint	=> $fs->mount_point($_),
-					type		=> $fs->type($_),
-				};
-			} sort {
-				-1 * (
-					length($fs->mount_point($a))
-						<=>
-					length($fs->mount_point($b))
-				)
-			} $fs->mounted_filesystems()
-		];
-		Debug 'Got filesystem informations.';
+		UpdateFileSystemInfo();
 	}
 
 	unless (
@@ -794,7 +963,7 @@ sub Initialize {
 		return 0;
 	}
 	Debug 'Scan data opened.';
-	push @{$Conf{'exclude-dir'}}, $Conf{data};
+	push @{$Conf{'exclude-dir'}}, qr|^\Q$Conf{data}\E|;
 
 	unless ($Summary{'time'}->{scan}->{start} = Time::HiRes::time()) {
 		Error 'Can not get hight resolution time.';
@@ -1015,10 +1184,18 @@ sub SetMoveDir {
 # @return HFS ファイルシステム上のパスかどうか
 sub IsHfsPath {
 	my	($path) = @_;
+	my	$rootDir = File::Spec->rootdir;
 
+	for (; $path ne $rootDir; $path = dirname($path)) {
+		foreach my $fs (@{$ActionSwitch{filesystem}}) {
+			if ($path eq $fs->{mountPoint}) {
+				return $fs->{type} eq 'hfs';
+			}
+		}
+	}
 	foreach my $fs (@{$ActionSwitch{filesystem}}) {
-		if (substr($path, 0, length($fs->{mountPoint})) eq $fs->{mountPoint}) {
-			return $fs->{type} eq 'hfs'; 
+		if ($rootDir eq $fs->{mountPoint}) {
+			return $fs->{type} eq 'hfs';
 		}
 	}
 
@@ -1049,7 +1226,10 @@ sub ReportSummary {
 	my	$startAt = localtime($Summary{'time'}->{start});
 	my	$period = $Conf{period};
 	my	$scanTime =
-			$Summary{'time'}->{scan}->{end} - $Summary{'time'}->{scan}->{start};
+			$Summary{'time'}->{scan}->{end}
+				-
+			$Summary{'time'}->{scan}->{start}
+	;
 	my	$scanTimeM = int($scanTime / 60);
 	my	$scanTimeS = int($scanTime - ($scanTimeM * 60));
 	my	$scanedData = $Summary{count}->{data}->{scaned};
@@ -1163,7 +1343,9 @@ sub DoesNeedScan {
 			$ActionSwitch{ScanDataHandle}, $path
 		);
 
-		return 1	if (!defined($lastScanedTime) or $lastScanedTime <= $mtime);
+		return 1	if (
+			!defined($lastScanedTime) or $lastScanedTime <= $mtime
+		);
 	}
 
 	# 同じバージョンで二回以上スキャンする必要はない。
@@ -1206,6 +1388,7 @@ sub DoesNeedScan {
 sub IsScanTargetDirectory {
 	my	($path) = @_;
 
+	$path = ReplaceFName($path);
 	!IsExcludeDir($path) and IsIncludeDir($path);
 }
 
@@ -1215,6 +1398,7 @@ sub IsScanTargetDirectory {
 sub IsScanTargetFile {
 	my	($path) = @_;
 
+	$path = ReplaceFName($path);
 	!IsExcludeFile($path) and IsIncludeFile($path);
 }
 
@@ -1286,7 +1470,9 @@ sub Scan {
 			ReportExcluded($target);
 			$target = undef;
 		}
-		elsif (!DoesNeedScan($target)) {
+		elsif (
+			!$Conf{dmg} || !IsDiskImageFile($target) and !DoesNeedScan($target)
+		) {
 			ReportChecked($target);
 			$target = undef;
 		}
@@ -1320,14 +1506,14 @@ sub Scan {
 				preprocess	=> $Conf{rsrc}
 					? sub {
 						my	@list = @_;
-
+	
 						$ActionSwitch{onHfsDir} = IsHfsPath($File::Find::dir);
 						$ActionSwitch{dirDepth}++;
 						return @list;
 					}
 					: sub {
 						my	@list = @_;
-
+	
 						$ActionSwitch{dirDepth}++;
 						return @list;
 					},
@@ -1361,7 +1547,9 @@ sub _Scan {
 		if (!IsScanTargetFile($path)) {
 			ReportExcluded($path);
 		}
-		elsif (!DoesNeedScan($path)) {
+		elsif (
+			!$Conf{dmg} || !IsDiskImageFile($path) and !DoesNeedScan($path)
+		) {
 			ReportChecked($path);
 		}
 		elsif (
@@ -1378,9 +1566,15 @@ sub _Scan {
 			ReportOkResult($path);
 		}
 		else {
-			my	$timeout = GetScanTimeout($path);
+			my	$timeout;
 			my	%result;
 
+			if ($Conf{dmg} && IsDiskImageFile($path)) {
+				$timeout = $Conf{timeout};
+			}
+			else {
+				$timeout = GetScanTimeout($path);
+			}
 			Debug 'timeout: %.3f s: %s', $timeout, EscapedPath($path);
 			eval {
 				local	$SIG{ALRM};
@@ -1443,6 +1637,86 @@ sub GetScanTimeout {
 }
 
 #--- clamd コミュニケーション ----
+# ファイルをマウントしてスキャン
+# @param $path	スキャン対象のファイル
+sub MountScan {
+	my	($path) = @_;
+	my	%result;
+	my	$mountPoint;
+	my	@umountArgs;
+	my	$check;
+
+	$check = defined(
+		$mountPoint = tempdir('ClamdOmitScan-XXXXXXXXXX',
+			CLEANUP => 1, DIR => File::Spec->tmpdir
+		)
+	);
+	if ($check) {
+		$mountPoint = realpath($mountPoint);
+		$check =
+			(
+				0 < scalar(@umountArgs = MountDiskImage($path, $mountPoint))
+					and
+				defined($umountArgs[0])
+			) ? 1 : 0
+		;
+		if ($check) {
+			my	$resultStr = '';
+			my	$realPath = ReplaceFName($path);
+
+			{
+				local	$Conf{recursive} = 1;
+				local	$ActionSwitch{onHfsDir};
+				local	$ActionSwitch{targetPrefix};
+
+				$ActionSwitch{targetPrefix} = {
+					prefix		=> $mountPoint,
+					prefixLen	=> length($mountPoint),
+					replace		=> $realPath,
+				};
+
+				find(
+					{
+						wanted		=> \&_Scan,
+						preprocess	=> $Conf{rsrc}
+							? sub {
+								my	@list = @_;
+			
+								$ActionSwitch{onHfsDir} = IsHfsPath(
+									$File::Find::dir
+								);
+								$ActionSwitch{dirDepth}++;
+								return @list;
+							}
+							: sub {
+								my	@list = @_;
+			
+								$ActionSwitch{dirDepth}++;
+								return @list;
+							},
+						postprocess => sub {
+							$ActionSwitch{dirDepth}--;
+						},
+					},
+					$mountPoint
+				);
+				$check = UnmountDiskImage(@umountArgs);
+
+				if ($ActionSwitch{targetPrefix}->{_infected}) {
+					MoveInfected($path);
+				}
+			}
+		}
+		else {
+			push @{$result{ERROR}}, [$path, 'Failed to mount.'];
+		}
+		rmdir $mountPoint;
+	}
+	unless ($check) {
+		push @{$result{ERROR}}, [$path, $!];
+	}
+}
+
 # ファイルスキャン
 # @param $path ファイルのパス
 # @return 次の形式のハッシュ
@@ -1456,7 +1730,10 @@ sub ScanFile {
 	my	($path) = @_;
 	my	%result = (OK => [], FOUND => [], ERROR => [], UNKNOWN => []);
 
-	if ($path =~ m/[\r\n]/) {
+	if ($Conf{dmg} and IsDiskImageFile($path)) {
+		MountScan($path);
+	}
+	elsif ($path =~ m/[\r\n]/) {
 		# 改行が入っていると clamd コマンドも改行されてしまい
 		# コマンドと一緒にファイル名を送れない。
 		# そこでこういうファイルだけ STREAM コマンドを使用する。
@@ -1555,7 +1832,11 @@ sub ScanStream {
 	my	$conn;
 	my	%result = (OK => [], FOUND => [], ERROR => [], UNKNOWN => []);
 
-	if ($Conf{'stream-max-length'} < $ActionSwitch{statCache}->[7]) {
+
+	if ($Conf{dmg} and IsDiskImageFile($path)) {
+		MountScan($path);
+	}
+	elsif ($Conf{'stream-max-length'} < $ActionSwitch{statCache}->[7]) {
 		push @{$result{ERROR}}, [$path, 'Too large.'];
 	}
 	else {
@@ -1934,7 +2215,8 @@ sub ParseVersion {
 		);
 
 		$version{dbBuildDateTime} = timelocal(
-			$sec, $min, $hour, $mday, $MonthName2Month{$mname}, $year - 1900
+			$sec, $min, $hour, $mday,
+			$MonthName2Month{$mname}, $year - 1900
 		);
 	}
 
@@ -1968,6 +2250,26 @@ sub MoveInfected {
 	my	($path) = @_;
 	my	$movedPath;
 
+	# ディスクイメージファイルの中のファイルのとき
+	if (
+		$ActionSwitch{targetPrefix}
+			and
+		substr($path, 0, $ActionSwitch{targetPrefix}->{prefixLen})
+			eq
+		$ActionSwitch{targetPrefix}->{prefix}
+			and
+		$path ne $ActionSwitch{targetPrefix}->{prefix}
+	) {
+		# CONTSCAN が指定されていなければディスクイメージファイル内のスキャンを
+		# 中止する。
+		$File::Find::prune = 1	unless($Conf{contscan});
+
+		# 制御側で移動をさせるため感染フラグを付ける
+		$ActionSwitch{targetPrefix}->{_infected} = 1;
+
+		return 1;
+	}
+
 	# 移動先ファイル名の決定
 	{
 		my	$moveDir = undef;
@@ -2120,6 +2422,261 @@ sub CopyFile {
 	return 1;
 }
 
+#--- ディスクイメージのマウント/アンマウント ----
+sub IsAbleDiskImage;
+sub IsDiskImageFile;
+sub MountDiskImage;
+sub UnmountDiskImage;
+
+sub _IsAbleDiskImage4darwin {
+	1;
+}
+
+sub _IsAbleDiskImage4linux {
+	my	$osver = eval {
+		local	$^W = 0;
+
+		`/bin/uname -r`
+	};
+
+	defined($osver) and $osver ge '2.4';
+}
+
+sub _IsAbleDiskImage4freebsd {
+	-f '/sbin/mdconfig' and -x '/sbin/mdconfig'
+}
+
+*IsAbleDiskImage = eval "*_IsAbleDiskImage4$^O";
+
+sub _IsDiskImageFile4darwin {
+	my	($path) = @_;
+	my	$retVal = 0;
+
+	if (-f $path and $path =~ m/.+\.(dmg|iso|img|cdr|vhd)$/i) {
+		$retVal = 1;
+	}
+
+	return $retVal;
+}
+
+sub _IsDiskImageFile {
+	my	($path) = @_;
+
+	if (-f $path and $path =~ m/.+\.iso$/i) {
+		return 1;
+	}
+	else {
+		return 0;
+	}
+}
+
+sub _IsDiskImageFile4linux;
+sub _IsDiskImageFile4freebsd;
+
+*_IsDiskImageFile4linux = *_IsDiskImageFile;
+*_IsDiskImageFile4freebsd = *_IsDiskImageFile;
+
+*IsDiskImageFile = eval "*_IsDiskImageFile4$^O";
+
+sub HdiutilError {
+	my	($ret, $diskImage) = @_;
+
+	$ret = $ret >> 8;
+	if ($! == -1) {
+		Error "%s: Cannot mount.", $diskImage;
+	}
+	else {
+		if ($ret == 1) {	# EPERM
+			Warn "%s: The refer operation cannot be performed.",
+				$diskImage
+			;
+		}
+		elsif ($ret == 6) {	# ENXIO
+			Error "%s: Device not configured.", $diskImage;
+		}
+		elsif ($ret == 13) {	# EACCES
+			Warn "%s: A particular access mode prevented the operation.",
+				$diskImage
+			;
+		}
+		elsif (
+			$ret == 16	# EBUSY
+		) {
+			$! = 127 & $ret;
+			Warn "%s: %s.", $diskImage, "$!";
+		}
+		elsif (
+			$ret == 22	# EINVAL
+				or
+			$ret == 27	# EFBIG
+				or
+			$ret == 80	# EAUTH
+		) {
+			$! = 127 & $ret;
+			Error "%s: %s.", $diskImage, "$!";
+		}
+		else {
+			Error "%s: mount process exits with %d.", $diskImage, $ret;
+		}
+	}
+}
+
+sub _MountDiskImage4darwin {
+	my	($diskImage, $mountPoint) = @_;
+	my	$ret;
+
+	Debug "mount %s to %s ...",  $diskImage, $mountPoint;
+	if ($diskImage =~ m/\.vhd$/) {
+		$ret = system(
+			qw(/usr/bin/hdiutil attach
+				-readonly -imagekey diskimage-class=CRawDiskImage
+				-quiet -nobrowse -mountpoint
+			),
+			$mountPoint, $diskImage
+		);
+	}
+	else {
+		$ret = system(
+			qw(
+				/usr/bin/hdiutil attach
+				-readonly -quiet -nobrowse -mountpoint
+			),
+			$mountPoint, $diskImage
+		);
+	}
+
+	if ($ret) {
+		HdiutilError($ret, ReplaceFName($diskImage));
+		system(qw(/usr/bin/hdiutil detach -quiet -force), $mountPoint);
+		return ();
+	}
+	else {
+		Debug "done.";
+		UpdateFileSystemInfo();
+		return ($mountPoint);
+	}
+}
+
+sub _UnmountDiskImage4darwin {
+	my	($mountPoint) = @_;
+	my	$ret;
+
+	Debug "unmount %s ...",  $mountPoint;
+	$ret = system(qw(/usr/bin/hdiutil detach -quiet -force), $mountPoint);
+	if ($ret) {
+		HdiutilError($ret, $mountPoint);
+	}
+	else {
+		Debug "done.";
+		UpdateFileSystemInfo();
+	}
+
+	return $ret == 0;
+}
+
+sub _MountDiskImage4linux {
+	my	($diskImage, $mountPoint) = @_;
+	my	$ret;
+	my	@retVal;
+	
+	Debug "mount %s to %s ...",  $diskImage, $mountPoint;
+	$ret = system(qw(mount -t iso9660 -o loop), $diskImage, $mountPoint);
+	if ($ret == 0) {
+		Debug "done.";
+		UpdateFileSystemInfo();
+		return ($mountPoint);
+	}
+	else {
+		Error "%s: %s", ReplaceFName($diskImage), "$!";
+		return ();
+	}
+}
+
+sub _UnmountDiskImage4linux {
+	my	($mountPoint) = @_;
+	my	$ret;
+
+	Debug "unmount %s ...",  $mountPoint;
+	$ret = system('umount', $mountPoint);
+	if ($ret == 0) {
+		Debug "done.";
+		UpdateFileSystemInfo();
+		return 1;
+	}
+	else {
+		Error "%s: %s", $mountPoint, "$!";
+		return 0;
+	}
+}
+
+sub _MountDiskImage4freebsd {
+	my	($diskImage, $mountPoint) = @_;
+	my	$devNum = 0;
+	my	$ret;
+	my	@retVal = ();
+
+	Debug "mount %s to %s ...",  $diskImage, $mountPoint;
+	while (-e "/dev/md$devNum") {
+		$devNum++;
+	}
+
+	$ret = system(
+		qw(/sbin/mdconfig -a -t vnode -f), $diskImage, '-u', $devNum
+	);
+	if ($ret == 0) {
+		$ret = system(
+			qw(/sbin/mount -t cd9660 -o ro), "/dev/md$devNum", $mountPoint
+		);
+		if ($ret == 0) {
+			Debug "done.";
+			UpdateFileSystemInfo();
+			@retVal = ("/dev/md$devNum");
+		}
+		else {
+			Error "%s: %s", ReplaceFName($diskImage), "$!";
+			system(qw(/sbin/mdconfig -d -u), $devNum);
+		}
+	}
+	else {
+		Error "%s: %s", ReplaceFName($diskImage), "$!";
+	}
+
+	return @retVal;
+}
+
+sub _UnmountDiskImage4freebsd {
+	my	($devFile) = @_;
+	my	$ret;
+
+	Debug "unmount %s ...",  $devFile;
+	if (defined($devFile) and $devFile =~ m'^/dev/md(\d+)$') {
+		my	$devNum = $1;
+
+		$ret = system('/sbin/umount', $devFile);
+		if ($ret == 0) {
+			$ret = system(qw(/sbin/mdconfig -d -u), $devNum);
+			if ($ret == 0) {
+				Debug "done.";
+				UpdateFileSystemInfo();
+			}
+			else {
+				Error "%s: %s", $devFile, "$!";
+			}
+		}
+		else {
+			Error "%s: %s", $devFile, "$!";
+		}
+	}
+	else {
+		$ret = -1;
+	}
+
+	return $ret == 0;
+}
+
+*MountDiskImage = eval "*_MountDiskImage4$^O";
+*UnmountDiskImage = eval "*_UnmountDiskImage4$^O";
+
 
 #-------------------------------------------------------------------------------
 # グローバル変数初期値
@@ -2175,6 +2732,8 @@ unless (exists $ENV{HOME} and defined($ENV{HOME})) {
 	'limit-timeout'	=> 3,
 	# リソースフォークをスキャンするかどうか
 	rsrc	=> 0,
+	# ディスクイメージ内をスキャンするかどうか
+	dmg	=> undef,
 
 	#--- clamdscan オプション ----
 	# 冗長モード
@@ -2210,7 +2769,7 @@ unless (exists $ENV{HOME} and defined($ENV{HOME})) {
 	# 感染ファイルのみ出力するかどうか
 	infected	=> 0,
 	# 感染ファイルをコピーするディレクトリ
-	'copy=s'	=> undef,
+	copy	=> undef,
 	# 感染発見時にベルを鳴らすかどうか
 	'bell'	=> 0,
 	# ディレクトリ最大深度
@@ -2313,7 +2872,7 @@ EOF
 		'scan-rsrc'	=> sub {
 			my	($name, $value) = @_;
 
-			if (defined $Sys::Filesystem::VERSION) {
+			if ($^O eq 'darwin' or defined $Sys::Filesystem::VERSION) {
 				$Conf{rsrc} = 1;
 			}
 			else {
@@ -2321,6 +2880,20 @@ EOF
 				exit $ExitStatus{UnsupportedOpts};
 			}
 		},
+		# ディスクイメージ内をスキャンするかどうか
+		'scan-dmg'	=> sub {
+			my	($name, $value) = @_;
+			my	$isAbleDiskImage = 0;
+
+			eval { $isAbleDiskImage = IsAbleDiskImage(); };
+			if ($isAbleDiskImage) {
+				$Conf{dmg} = 1;
+			}
+			else {
+				Error "Sorry, %s is not supported on this system.", $name;
+				exit $ExitStatus{UnsupportedOpts};
+			}
+		},
 	},
 
 	# clamdscan オプション
@@ -2371,13 +2944,29 @@ EOF
 			# ディレクトリを再帰的にスキャンするかどうか
 			'recursive|r'	=> \$Conf{recursive},
 			# スキャン対象外ファイル
-			'exclude=s'	=> $Conf{exclude},
+			'exclude=s'	=> sub {
+				my	($name, $value) = @_;
+
+				push @{$Conf{exclude}}, qr|$value|;
+			},
 			# スキャン対象外ディレクトリ
-			'exclude-dir=s'	=> $Conf{'exclude-dir'},
+			'exclude-dir=s'	=> sub {
+				my	($name, $value) = @_;
+
+				push @{$Conf{'exclude-dir'}}, qr|$value|;
+			},
 			# スキャン対象ファイル
-			'include=s'	=> $Conf{include},
+			'include=s'	=> sub {
+				my	($name, $value) = @_;
+
+				push @{$Conf{include}}, qr|$value|;
+			},
 			# スキャン対象ディレクトリ
-			'include-dir=s'	=> $Conf{'include-dir'},
+			'include-dir=s'	=> sub {
+				my	($name, $value) = @_;
+
+				push @{$Conf{'include-dir'}}, qr|$value|;
+			},
 			# 感染ファイルのみを出力するかどうか
 			'infected'	=> \$Conf{infected},
 			# 感染ファイルをコピーするディレクトリ
@@ -2559,6 +3148,14 @@ EOF
 
 	# HFS フォーマットのファイルシステムのディレクトリ上かどうか
 	onHfsDir	=> 0,
+
+	# スキャン対象のファイルの前方置換スペック
+	#	{
+	#		prefix => 前方の文字列,
+	#		prefixLen => 前方の文字列の長さ,
+	#		replace => 置換文字列
+	#	}
+	targetPrefix	=> undef,
 );
 
 #-------------------------------------------------------------------------------
@@ -2785,12 +3382,12 @@ See L<clamd.conf(5)/StreamMaxLength>.
 When this option is not specfied, B<ClamdOmmitScan.pl> uses C<StreamMaxLength>
 value, if it is written in F<clamd.conf>.
 
-B<WARNING>: B<ClamOmitScan.pl> might die when I<length> is larger than the value
-clamd actually used. They are the following cases.
+B<WARNING>: B<ClamdOmitScan.pl> might die when I<length> is larger than the
+value clamd actually used. They are the following cases.
 
 =over 2
 
-* B<ClamOmitScan.pl> uses STREAM clamd command.
+* B<ClamdOmitScan.pl> uses STREAM clamd command.
 
 * AND the size of the file to be scanned is larger than I<length>.
 
@@ -2877,10 +3474,31 @@ DEFAULT: 3
 
 Scan a resource fork of each file on HFS filesystem.
 
-This option requires perl module L<Sys::Filesystem|Sys::Filesystem>.
+When your system is not Darwin(Mac OS X),
+this option requires perl module L<Sys::Filesystem|Sys::Filesystem>.
 
 DEFAULT: (no)
 
+=item --scan-dmg
+
+Treat disk image files as directory. B<ClamdOmitScan> mounts a disk image file
+to temporary directory, and scans files under it. This option is supported on
+Darwin(Mac OS X), Linux, FreeBSD.
+
+If your system is Linux, kernel version must be equal or greater than 2.4.
+If your system is FreeBSD, F</sbin/mdconfig> must be installed.
+Both of Linux and FreeBSD require the perl module L<Sys::Filesystem|Sys::Filesystem>.
+
+Supported type of disk image file are the following:
+
+    Darwin(Mac OS X): .iso, .dmg, .img, .cdr, .vhd(Virual PC disk image)
+    the other OSs: .iso
+
+NOTE: Disk image files for which agreement of license is necessary or encrypted
+disk image files are supported on only TTY environments.
+
+DEFALUT: (no)
+
 =back
 
 =head2 CLAMDSCAN OPTIONS
@@ -2929,7 +3547,7 @@ C<ClamdOmitScan.pl file>
 
 =over
 
-C<ClamOmitScan.pl>
+C<ClamdOmitScan.pl>
 
 =back
 
@@ -2937,7 +3555,7 @@ C<ClamOmitScan.pl>
 
 =over
 
-C<ClamOmitScan.pl -r /home>
+C<ClamdOmitScan.pl -r /home>
 
 =back
 
@@ -2945,7 +3563,7 @@ C<ClamOmitScan.pl -r /home>
 
 =over
 
-C<ClamOmitScan.pl --data=/var/ClamdOmitScan/scanData>
+C<ClamdOmitScan.pl --data=/var/ClamdOmitScan/scanData>
 
 =back
 
@@ -2953,7 +3571,7 @@ C<ClamOmitScan.pl --data=/var/ClamdOmitScan/scanData>
 
 =over
 
-C<ClamOmitScan.pl --data=/var/ClamdOmitScan/scanData --maintain-scan-data>
+C<ClamdOmitScan.pl --data=/var/ClamdOmitScan/scanData --maintain-scan-data>
 
 =back
 
